import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        this._amount = parseInt(window.location.search.substr(1)) || 10;
        this._count = Math.pow(this._amount, 3);

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        domWebGL.appendChild(renderer.domElement);
        //renderer.outputEncoding = THREE.sRGBEncoding; // 유무차이 
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0xffffff);
        this._scene = scene;

        this._setupCamera();
        //this._setupControls();
        //this._setupLight();
        this._setupModel();
        //this._setupHelper();
        
        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupModel() {
        const loader = new THREE.BufferGeometryLoader()
        loader.load("models/json/suzanne_buffergeometry.json", (geometry) => {
            geometry.computeVertexNormals();
            geometry.scale(0.5, 0.5, 0.5);

            const material = new THREE.MeshNormalMaterial();
            const mesh = new THREE.InstancedMesh(geometry, material, this._count);
            mesh.instanceMatrix.setUsage(THREE.DynamicDrawUsage);
            this._scene.add(mesh);

            this._mesh = mesh;

            this._setupGUI();
        });
    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);
        this._stats = stats;

        const gui = new GUI();
        gui.add(this._mesh, "count", 0, this._count);
    }

    _setupCamera() {    
        const camera = new THREE.PerspectiveCamera(60, window.innerWidth / window.innerHeight, 1, 100);
        camera.position.set(this._amount * 0.9, this._amount * 0.9, this._amount * 0.9);
        camera.lookAt(0, 0, 0);

        this._camera = camera;
    }

    _setupControls() {
        //.
    }

    _setupLight() {
        //.
    }

    update() {
        if(this._mesh) {
            this._stats.update();

            const dummy = new THREE.Object3D();
            const time = Date.now() * 0.001;

            this._mesh.rotation.x = Math.sin(time / 4);
            this._mesh.rotation.y = Math.sin(time / 2);

            let i = 0;
            const offset = (this._amount - 1) / 2;
            for(let x=0; x<this._amount; x++) {
                for(let y=0; y<this._amount; y++) {
                    for(let z=0; z<this._amount; z++) {
                        dummy.position.set(offset-x, offset-y, offset-z);
                        dummy.rotation.y = (Math.sin(x/4+time) + Math.sin(y/4+time) + Math.sin(z/4+time));
                        dummy.rotation.z = dummy.rotation.y * 2;

                        dummy.updateMatrix();

                        this._mesh.setMatrixAt(i++, dummy.matrix);
                    }
                }
            }

            this._mesh.instanceMatrix.needsUpdate = true;
        }
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this._camera.aspect = aspect;
        this._camera.updateProjectionMatrix();

        this._renderer.setPixelRatio( window.devicePixelRatio );
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    (new App()).render();
}